@@ -2,10 +2,12 @@ package inject
 
 import (
 	"fmt"
+	"reflect"
 	"sort"
 	"strings"
 
 	"github.com/jaegertracing/jaeger-operator/pkg/config/ca"
+	"github.com/jaegertracing/jaeger-operator/pkg/config/otelconfig"
 
 	log "github.com/sirupsen/logrus"
 	"github.com/spf13/viper"
@@ -54,10 +56,16 @@ func Sidecar(jaeger *v1.Jaeger, dep *appsv1.Deployment) *appsv1.Deployment {
 		logFields.Trace("deployment is assigned to a different Jaeger instance, skipping sidecar injection")
 		return dep
 	}
-
 	decorate(dep)
+	hasAgent, agentContainerIndex := HasJaegerAgent(dep)
 	logFields.Debug("injecting sidecar")
-	dep.Spec.Template.Spec.Containers = append(dep.Spec.Template.Spec.Containers, container(jaeger, dep))
+	if hasAgent { // This is an update
+		dep.Spec.Template.Spec.Containers[agentContainerIndex] = container(jaeger, dep)
+	} else {
+		dep.Spec.Template.Spec.Containers = append(dep.Spec.Template.Spec.Containers, container(jaeger, dep))
+
+	}
+
 	jaegerName := util.Truncate(jaeger.Name, 63)
 
 	if dep.Labels == nil {
@@ -69,40 +77,33 @@ func Sidecar(jaeger *v1.Jaeger, dep *appsv1.Deployment) *appsv1.Deployment {
 	return dep
 }
 
-// Desired determines whether a sidecar is desired, based on the annotation from both the deployment and the namespace
-func Desired(dep *appsv1.Deployment, ns *corev1.Namespace) bool {
-	logger := log.WithFields(log.Fields{
-		"namespace":  dep.Namespace,
-		"deployment": dep.Name,
-	})
-	_, depExist := dep.Annotations[Annotation]
-	_, nsExist := ns.Annotations[Annotation]
-
-	if depExist {
-		logger.Debug("annotation present on deployment")
-		return true
-	}
-
-	if nsExist {
-		logger.Debug("annotation present on namespace")
-		return true
-	}
-
-	return false
-}
-
 // Needed determines whether a pod needs to get a sidecar injected or not
 func Needed(dep *appsv1.Deployment, ns *corev1.Namespace) bool {
-	if !Desired(dep, ns) {
+	_, depExist := dep.Annotations[Annotation]
+	_, nsExist := ns.Annotations[Annotation]
+	if !depExist && !nsExist {
+		log.WithFields(log.Fields{
+			"namespace":  dep.Namespace,
+			"deployment": dep.Name,
+		}).Trace("annotation not present, not injecting")
 		return false
 	}
-
 	// do not inject jaeger due to port collision
 	// do not inject if deployment's Annotation value is false
 	if dep.Labels["app"] == "jaeger" {
 		return false
 	}
-	return !HasJaegerAgent(dep)
+
+	hasAgent, _ := HasJaegerAgent(dep)
+
+	if hasAgent {
+		// has already a sidecar injected and managed by the operator
+		// return true because could require an update.
+		_, hasLabel := dep.Labels[Label]
+		return hasLabel
+	}
+	// If no agent but has annotations
+	return true
 }
 
 // Select a suitable Jaeger from the JaegerList for the given Pod, or nil of none is suitable
@@ -121,18 +122,38 @@ func Select(target *appsv1.Deployment, ns *corev1.Namespace, availableJaegerPods
 		return jaeger
 	}
 
-	if (strings.EqualFold(jaegerNameDep, "true") || strings.EqualFold(jaegerNameNs, "true")) &&
-		len(availableJaegerPods.Items) == 1 {
-		// if there's only *one* jaeger within this namespace, then that's what
-		// we'll use -- otherwise, we should just not inject, as it's not clear which
-		// jaeger instance to use!
-		// first, we make sure we normalize the name:
-		jaeger := &availableJaegerPods.Items[0]
-		return jaeger
+	if strings.EqualFold(jaegerNameDep, "true") || strings.EqualFold(jaegerNameNs, "true") {
+		// If there is only *one* available instance in all watched namespaces
+		// then that's what we'll use
+		if len(availableJaegerPods.Items) == 1 {
+			jaeger := &availableJaegerPods.Items[0]
+			return jaeger
+		}
+		// If there is more than one available instance in all watched namespaces
+		// then we should find if there is only *one* on the same namespace
+		// if that is the case. we should use it.
+		instancesInNamespace := getJaegerFromNamespace(target.Namespace, availableJaegerPods)
+		if len(instancesInNamespace) == 1 {
+			jaeger := instancesInNamespace[0]
+			return jaeger
+		}
+		// At this point, we have more than one instance that could be used to inject
+		// we should just not inject, as it's not clear which one should be used.
 	}
 	return nil
 }
 
+func getJaegerFromNamespace(namespace string, jaegers *v1.JaegerList) []*v1.Jaeger {
+	var instances []*v1.Jaeger
+	for _, p := range jaegers.Items {
+		if p.Namespace == namespace {
+			// matched the namespace!
+			instances = append(instances, &p)
+		}
+	}
+	return instances
+}
+
 func getJaeger(name string, jaegers *v1.JaegerList) *v1.Jaeger {
 	for _, p := range jaegers.Items {
 		if p.Name == name {
@@ -146,18 +167,14 @@ func getJaeger(name string, jaegers *v1.JaegerList) *v1.Jaeger {
 func container(jaeger *v1.Jaeger, dep *appsv1.Deployment) corev1.Container {
 	args := append(jaeger.Spec.Agent.Options.ToArgs())
 
-	if len(util.FindItem("--reporter.type=", args)) == 0 {
-		args = append(args, "--reporter.type=grpc")
-
-		// we only add the grpc host if we are adding the reporter type and there's no explicit value yet
-		if len(util.FindItem("--reporter.grpc.host-port=", args)) == 0 {
-			args = append(args, fmt.Sprintf("--reporter.grpc.host-port=dns:///%s.%s.svc:14250", service.GetNameForHeadlessCollectorService(jaeger), jaeger.Namespace))
-		}
+	// we only add the grpc host if we are adding the reporter type and there's no explicit value yet
+	if len(util.FindItem("--reporter.grpc.host-port=", args)) == 0 {
+		args = append(args, fmt.Sprintf("--reporter.grpc.host-port=dns:///%s.%s.svc:14250", service.GetNameForHeadlessCollectorService(jaeger), jaeger.Namespace))
 	}
 
 	// Enable tls by default for openshift platform
 	if viper.GetString("platform") == v1.FlagPlatformOpenShift {
-		if len(util.FindItem("--reporter.type=grpc", args)) > 0 && len(util.FindItem("--reporter.grpc.tls.enabled=true", args)) == 0 {
+		if len(util.FindItem("--reporter.grpc.tls.enabled=true", args)) == 0 {
 			args = append(args, "--reporter.grpc.tls.enabled=true")
 			args = append(args, fmt.Sprintf("--reporter.grpc.tls.ca=%s", ca.ServiceCAPath))
 		}
@@ -191,15 +208,24 @@ func container(jaeger *v1.Jaeger, dep *appsv1.Deployment) corev1.Container {
 
 	// Use only the agent common spec for volumes and mounts.
 	// We don't want to mount all Jaeger internal volumes into user's deployments
-	volumesAndMountsSpec := jaeger.Spec.Agent.JaegerCommonSpec
+	volumesAndMountsSpec := &jaeger.Spec.Agent.JaegerCommonSpec
+	otelConf, err := jaeger.Spec.Agent.Config.GetMap()
+	if err != nil {
+		jaeger.Logger().WithField("error", err).
+			WithField("component", "agent").
+			Errorf("Could not parse OTEL config, config map will not be created")
+	} else if otelconfig.ShouldCreate(jaeger, jaeger.Spec.Agent.Options, otelConf) {
+		otelconfig.Update(jaeger, "agent", volumesAndMountsSpec, &args)
+	}
 
-	ca.Update(jaeger, &volumesAndMountsSpec)
-	ca.AddServiceCA(jaeger, &volumesAndMountsSpec)
+	ca.Update(jaeger, volumesAndMountsSpec)
+	ca.AddServiceCA(jaeger, volumesAndMountsSpec)
 
 	// ensure we have a consistent order of the arguments
 	// see https://github.com/jaegertracing/jaeger-operator/issues/334
 	sort.Strings(args)
 
+	dep.Spec.Template.Spec.ImagePullSecrets = util.RemoveDuplicatedImagePullSecrets(append(dep.Spec.Template.Spec.ImagePullSecrets, jaeger.Spec.Agent.ImagePullSecrets...))
 	dep.Spec.Template.Spec.Volumes = util.RemoveDuplicatedVolumes(append(dep.Spec.Template.Spec.Volumes, volumesAndMountsSpec.Volumes...))
 	return corev1.Container{
 		Image: util.ImageName(jaeger.Spec.Agent.Image, "jaeger-agent-image"),
@@ -248,8 +274,9 @@ func container(jaeger *v1.Jaeger, dep *appsv1.Deployment) corev1.Container {
 				Name:          "admin-http",
 			},
 		},
-		Resources:    commonSpec.Resources,
-		VolumeMounts: volumesAndMountsSpec.VolumeMounts,
+		Resources:       commonSpec.Resources,
+		SecurityContext: jaeger.Spec.Agent.SidecarSecurityContext,
+		VolumeMounts:    volumesAndMountsSpec.VolumeMounts,
 	}
 }
 
@@ -279,7 +306,7 @@ func decorate(dep *appsv1.Deployment) {
 			if !hasEnv(envVarPropagation, dep.Spec.Template.Spec.Containers[i].Env) {
 				dep.Spec.Template.Spec.Containers[i].Env = append(dep.Spec.Template.Spec.Containers[i].Env, corev1.EnvVar{
 					Name:  envVarPropagation,
-					Value: "jaeger,b3",
+					Value: "jaeger,b3,w3c",
 				})
 			}
 		}
@@ -329,13 +356,25 @@ func CleanSidecar(instanceName string, deployment *appsv1.Deployment) {
 }
 
 // HasJaegerAgent checks whether deployment has Jaeger Agent container
-func HasJaegerAgent(dep *appsv1.Deployment) bool {
+func HasJaegerAgent(dep *appsv1.Deployment) (bool, int) {
 	// this pod is annotated, it should have a sidecar
 	// but does it already have one?
-	for _, container := range dep.Spec.Template.Spec.Containers {
+	for i, container := range dep.Spec.Template.Spec.Containers {
 		if container.Name == "jaeger-agent" { // we don't labels/annotations on containers, so, we rely on its name
-			return true
+			return true, i
 		}
 	}
-	return false
+	return false, -1
+}
+
+// EqualSidecar check if two deployments sidecar are equal
+func EqualSidecar(dep, oldDep *appsv1.Deployment) bool {
+	depHasAgent, depAgentIndex := HasJaegerAgent(dep)
+	oldDepHasAgent, oldDepIndex := HasJaegerAgent(oldDep)
+	if depHasAgent != oldDepHasAgent {
+		return false
+	}
+	depContainer := dep.Spec.Template.Spec.Containers[depAgentIndex]
+	oldDepContainer := oldDep.Spec.Template.Spec.Containers[oldDepIndex]
+	return reflect.DeepEqual(depContainer, oldDepContainer)
 }
